#define F_CPU 16000000L
#include <avr/io.h>
#include <util/delay.h>
#include "UART0.h"

void ADC_init(unsigned char channel) {
  ADMUX |= (1 << REFS0);    // AVCC를 기준 전압으로 선택
  // 채널 선택. 8~15번 채널은 ADMUX 이외에 ADCSRB 레지스터 설정 필요
  ADMUX = ((ADMUX & 0xE0) | (channel & 0x07));
  if (channel > 8) ADCSRB |= (1 << MUX5);

  // 분주비를 128로 설정
  ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);

  ADCSRA |= (1 << ADEN);    // ADC 활성화
}

int read_ADC(void) {
  ADCSRA |= (1 << ADSC);    // 변환 시작
  while (!(ADCSRA & (1 << ADIF))); // 변환 종료 대기

  return ADC;       // 10비트 값을 반환
}

int main(void) {
  int read;

  UART0_init();       // UART 통신 초기화
  ADC_init(0);        // AD 변환기 초기화

  while (1) {
    read = read_ADC();      // 가변저항 읽기

    UART0_print(read);
    UART0_write('\n');

    _delay_ms(1000);      // 1초에 한 번 읽음
  }

  return 0;
}
