#define F_CPU 16000000L
#include <avr/io.h>
#include "millis2560.h"

#define set_bit(value, bit) ( _SFR_BYTE(value) |= _BV(bit) )
#define clear_bit(value, bit) ( _SFR_BYTE(value) &= ~_BV(bit) )

void shift_clock(void) {
  set_bit(PORTC, 5);      // 이동 클록을 HIGH로
  clear_bit(PORTC, 5);      // 이동 클록을 LOW로
}

void latch_clock(void) {
  set_bit(PORTC, 6);      // 래치 클록을 HIGH로
  clear_bit(PORTC, 6);      // 래치 클록을 LOW로
}

void byte_data_write(uint8_t data) {  // 1바이트 데이터 출력
  for (uint8_t i = 0; i < 8; i++) {
    // MSB부터 1비트씩 출력
    if (data & 0b10000000) set_bit(PORTC, 7);
    else clear_bit(PORTC, 7);

    shift_clock();      // 1비트 출력 후 비트 이동
    data = data << 1;       // 다음 출력할 비트를 MSB로 이동
  }
}

int main(void) {
  // 행 데이터로 선택된 LED만 1의 값을 가짐
  byte row_data[] = {
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,     // 스크롤을 위한 추가
    0b00111100, 0b01000010, 0b10010101, 0b10100001,
    0b10100001, 0b10010101, 0b01000010, 0b00111100,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00  // 스크롤을 위한 추가
  };

  // 열 데이터로 선택된 열만 1로 표시되어 있고 실제로는 반전하여 출력
  byte col_data[8] = {
    0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80
  };

  // 데이터 PC7, 이동 클록 PC5, 래치 클록 PC6
  DDRC |= 0b11100000;
  PORTC |= 0b11100000;

  millis2560_init();

  int start_index = 0;      // 표시할 영역의 인덱스
  unsigned long time_p = millis2560(), time_c;

  while (1) {
    time_c = millis2560();
    if (time_c - time_p >= 300) {
      time_p = time_c;
      start_index = (start_index + 1) % 16;
    }
    for (int i = 0; i < 8; i++) {
      // 열 데이터 -> 행 데이터 순으로 출력하며 열 데이터는 반전해서 출력
      byte_data_write(~col_data[i]);
      byte_data_write(row_data[i + start_index]);

      latch_clock();      // 2바이트 전달 후 실제 출력 발생

      _delay_ms(2);       // LED가 완전히 켜지는 시간
    }
  }

  return 0;
}
