#define F_CPU 16000000UL

#include <avr/io.h>
#include <util/delay.h>

#include "UART0.h"
#include "I2C_RTC.h"

int main(void) {
  I2C_init();           // I2C 초기화
  UART0_init();         // UART 초기화

  uint8_t address = 0x68;     // RTC 모듈의 I2C 주소

  // 초, 분, 시, 요일, 일, 월, 년 (2018년 5월 1일 화요일 12시 00분 00초)
  uint8_t data[] = {00, 00, 12, 3, 1, 5, 18};

  // RTC 모듈에 시간 설정
  I2C_start();        // I2C 시작
  I2C_transmit(address << 1);     // I2C 주소 전송. 쓰기 모드
  // RTC에 데이터를 기록할 메모리 시작 주소 전송
  I2C_transmit(0);

  UART0_print("* Setting RTC module...\n");
  for (int i = 0; i < 7; i++) {
    I2C_transmit(decimal_to_bcd(data[i]));  // 시간 설정
    UART0_print(i + 1);
    UART0_print("th byte written...\n");
  }

  I2C_stop();         // I2C 정지

  _delay_ms(2000);      // 2초 대기

  I2C_start();        // I2C 시작
  I2C_transmit(address << 1);     // I2C 주소 전송. 쓰기 모드
  // RTC에서 데이터를 읽어올 메모리 시작 주소 전송
  I2C_transmit(0);
  I2C_stop();         // I2C 정지

  I2C_start();        // I2C 읽기 모드로 다시 시작
  I2C_transmit( (address << 1) + 1 ); // I2C 주소 전송. 읽기 모드

  UART0_print("* Time/Date Retrieval...\n");
  UART0_print(" Second : ");    // 초
  UART0_print(bcd_to_decimal(I2C_receive_ACK()));
  UART0_write('\n');
  UART0_print(" Minute : ");    // 분
  UART0_print(bcd_to_decimal(I2C_receive_ACK()));
  UART0_write('\n');
  UART0_print(" Hour : ");    // 시
  UART0_print(bcd_to_decimal(I2C_receive_ACK()));
  UART0_write('\n');
  UART0_print(" Day of Week : "); // 요일
  UART0_print(bcd_to_decimal(I2C_receive_ACK()));
  UART0_write('\n');
  UART0_print(" Day : ");     // 일
  UART0_print(bcd_to_decimal(I2C_receive_ACK()));
  UART0_write('\n');
  UART0_print(" Month : ");   // 월
  UART0_print(bcd_to_decimal(I2C_receive_ACK()));
  UART0_write('\n');
  UART0_print(" Year : ");    // 년
  UART0_print(bcd_to_decimal(I2C_receive_NACK()));
  UART0_write('\n');

  I2C_stop();         // I2C 정지

  while (1) {
  }

  return 0;
}
