#include "TextLCD.h"

void LCD_pulse_enable(void) { 		// 하강 에지에서 동작
	set_bit(PORTL, E_PIN); 			// E를 HIGH로
	_delay_us(1);
	clear_bit(PORTL, E_PIN); 		// E를 LOW로
	_delay_ms(1);
}

void LCD_write_data(uint8_t data) {
	set_bit(PORTL, RS_PIN); 		// 문자 출력에서 RS는 1

	uint8_t temp = reverse_bit_order(data);

	PORTL = (PORTL & 0xF0) | (temp & 0x0F);
	LCD_pulse_enable(); 			// 문자 데이터 상위 4비트 출력

	PORTL = (PORTL & 0xF0) | (temp >> 4);
	LCD_pulse_enable(); 			// 문자 데이터 하위 4비트 출력

	_delay_ms(2);
}

void LCD_write_command(uint8_t command) {
	clear_bit(PORTL, RS_PIN); 		// 명령어 실행에서 RS는 0

	uint8_t temp = reverse_bit_order(command);

	PORTL = (PORTL & 0xF0) | (temp & 0x0F);
	LCD_pulse_enable(); 			// 명령어 데이터 상위 4비트 출력

	PORTL = (PORTL & 0xF0) | (temp >> 4);
	LCD_pulse_enable(); 			// 명령어 데이터 하위 4비트 출력

	_delay_ms(2);
}

void LCD_clear(void) {
	LCD_write_command(COMMAND_CLEAR_DISPLAY);
	_delay_ms(2);
}

uint8_t reverse_bit_order(uint8_t data) {
	//data = ((data >> 1) & 0x55) | ((data << 1) & 0xaa);
	uint8_t result = 0;
	if (data & (1 << 0)) result |= (0x80 >> 0);
	if (data & (1 << 1)) result |= (0x80 >> 1);
	if (data & (1 << 2)) result |= (0x80 >> 2);
	if (data & (1 << 3)) result |= (0x80 >> 3);
	if (data & (1 << 4)) result |= (0x80 >> 4);
	if (data & (1 << 5)) result |= (0x80 >> 5);
	if (data & (1 << 6)) result |= (0x80 >> 6);
	if (data & (1 << 7)) result |= (0x80 >> 7);

	return result;
}

void LCD_init(void) {
	_delay_ms(50); 			// 초기 구동 시간

	// 연결 핀을 출력으로 설정
	DDRL |= 0x0F;				// 데이터 핀
	DDRL |= (1 << RS_PIN) | (1 << E_PIN);	// 제어 핀

	LCD_write_command(0x02); 		// 4비트 모드 추가 명령
	LCD_write_command(COMMAND_4_BIT_MODE); 	// 4비트 모드

	// display on/off control
	// 화면 on, 커서 off, 커서 깜빡임 off
	uint8_t command = 0x08 | (1 << COMMAND_DISPLAY_ON_OFF_BIT);
	LCD_write_command(command);

	LCD_clear(); 				// 화면 지움

	// Entry Mode Set
	// 출력 후 커서를 오른쪽으로 옮김, 즉, DDRAM의 주소가 증가하며 화면 이동은 없음
	LCD_write_command(0x06);
}

void LCD_write_string(char *string) {
	uint8_t i;
	for (i = 0; string[i]; i++) 		// 종료 문자를 만날 때까지
	LCD_write_data(string[i]); 		// 문자 단위 출력
}

void LCD_goto_XY(uint8_t row, uint8_t col) {
	col %= 16; 				// [0 15]
	row %= 2; 				// [0 1]

	// 첫째 라인 시작 주소는 0x00, 둘째 라인 시작 주소는 0x40
	uint8_t address = (0x40 * row) + col;
	uint8_t command = 0x80 + address;

	LCD_write_command(command); 	// 커서 이동
}
